package dev.fitko.fitconnect.api.domain.model.reply.replychannel;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.nimbusds.jose.jwk.JWK;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

@Data
@NoArgsConstructor
@AllArgsConstructor
@JsonInclude(JsonInclude.Include.NON_NULL)
public class ReplyChannel {

    @JsonProperty("deMail")
    private DeMail deMail;

    @JsonProperty("elster")
    private Elster elster;

    @JsonProperty("eMail")
    private Email eMail;

    @JsonProperty("fink")
    private Fink fink;

    @JsonProperty("fitConnect")
    private FitConnect fitConnect;

    ReplyChannel(final Fink fink) {
        this.fink = fink;
    }

    ReplyChannel(final Email eMail) {
        this.eMail = eMail;
    }

    ReplyChannel(final DeMail deMail) {
        this.deMail = deMail;
    }

    ReplyChannel(final Elster elster) {
        this.elster = elster;
    }

    ReplyChannel(final FitConnect fitConnect) {
        this.fitConnect = fitConnect;
    }

    public static ReplyChannel fromFink(final String finkPostboxRef, final String host) {
        return new ReplyChannel(new Fink(finkPostboxRef, host));
    }

    public static ReplyChannel fromEmailWithPgp(final String address, final String pgpPublicKey) {
        return new ReplyChannel(new Email(address, true, pgpPublicKey));
    }

    public static ReplyChannel fromEmail(final String address) {
        return new ReplyChannel(new Email(address, false, null));
    }

    public static ReplyChannel fromDeMail(final String address) {
        return new ReplyChannel(new DeMail(address));
    }

    public static ReplyChannel fromElster(final String accountId, final String deliveryTicket, final String reference) {
        return new ReplyChannel(new Elster(accountId, deliveryTicket, reference));
    }

    public static ReplyChannel fromFitConnect(final JWK encryptionKey, final List<String> processStandards) {
        return new ReplyChannel(new FitConnect(EncryptionPublicKey.fromJwk(encryptionKey), processStandards));
    }

}

